# coding: UTF-8

import mailup.mailup_exceptions as mailup_exceptions

from mailup.logger import LoggerSingleton
from mailup.utils import (
    check_list_data_dict, check_group_data_dict, check_recipient_data_dict, check_message_data_dict,
    check_tag_data_dict,
)


class MailUpObjectProvider(object):
    client = None
    logger = None

    def __init__(self, client, logger=None):
        # CLIENT INITIALISATION
        self.client = client

        # LOGGER INITIALISATION
        if logger:
            self.logger = logger
        else:
            self.logger = LoggerSingleton()

        super(MailUpObjectProvider, self).__init__()

    # LIST PROVIDER METHODS
    def create_list(self, list_data_dict):

        # next check can't placed in check_list_data_dict
        missing_parameters = dict()
        if 'Name' not in list_data_dict:
            missing_parameters['Name'] = None
        if 'owneremail' not in list_data_dict:
            missing_parameters['owneremail'] = None
        if 'replyto' not in list_data_dict:
            missing_parameters['replyto'] = None
        if missing_parameters:
            raise mailup_exceptions.InvalidListConfigurationException(missing_parameters)

        # check list_data_dict
        check_list_data_dict(list_data_dict)

        # Default List parameter
        if 'useDefaultSettings' not in list_data_dict:
            list_data_dict['useDefaultSettings'] = False
        if 'idSettings' not in list_data_dict:
            list_data_dict['idSettings'] = 1
        if 'business' not in list_data_dict:
            list_data_dict['business'] = True
        if 'Customer' not in list_data_dict:
            list_data_dict['Customer'] = True
        if 'scope' not in list_data_dict:
            list_data_dict['scope'] = "newsletters"
        if 'format' not in list_data_dict:
            list_data_dict['format'] = "html"
        if 'copyTemplate' not in list_data_dict:
            list_data_dict['copyTemplate'] = True
        if 'copyWebhooks' not in list_data_dict:
            list_data_dict['copyWebhooks'] = True
        if 'charset' not in list_data_dict:
            list_data_dict['charset'] = "UTF-8"
        if 'public' not in list_data_dict:
            list_data_dict['public'] = True
        if 'tracking' not in list_data_dict:
            list_data_dict['tracking'] = True
        if 'optout_type' not in list_data_dict:
            list_data_dict['optout_type'] = 0
        if 'sendemailoptout' not in list_data_dict:
            list_data_dict['sendemailoptout'] = False
        if 'frontendform' not in list_data_dict:
            list_data_dict['frontendform'] = True
        if 'headerxabuse' not in list_data_dict:
            list_data_dict['headerxabuse'] = "Please report abuse here: http://[host]/p"
        if 'kbmax' not in list_data_dict:
            list_data_dict['kbmax'] = 100
        if 'headerlistunsubscriber' not in list_data_dict:
            list_data_dict['headerlistunsubscriber'] = "<[listunsubscribe]>,<[mailto_uns]>"
        if 'multipart_text' not in list_data_dict:
            list_data_dict['multipart_text'] = True
        if 'subscribedemail' not in list_data_dict:
            list_data_dict['subscribedemail'] = True
        if 'sendconfirmsms' not in list_data_dict:
            list_data_dict['sendconfirmsms'] = False
        list_id = self.client.create_list(
            data_dict=list_data_dict
        )
        self.logger.info('List with id {list_id} created successfully'.format(list_id=list_id))
        return self.get_list(list_id)

    def get_list(self, list_id):
        all_lists = self.all_lists()
        for current_list in all_lists:
            if current_list.id == list_id:
                self.logger.debug('List with id {list_id} found'.format(list_id=list_id))
                return current_list
        self.logger.error('List with id {list_id} not found'.format(list_id=list_id))
        raise mailup_exceptions.ListNotFoundException(list_id)

    def all_lists(self):
        from mailup.components import List

        paginated_lists_data_dict = self.client.read_lists()
        all_lists = []
        lists_data_dict = paginated_lists_data_dict['Items']
        for list_data_dict in lists_data_dict:
            new_list = List(
                data_dict=list_data_dict,
                mailup_client=self.client,
                logger=self.logger
            )
            all_lists.append(new_list)
        self.logger.debug('{count} Lists founds'.format(count=len(all_lists)))
        return all_lists

    # GROUP PROVIDER METHODS
    def create_group(self, group_data_dict):
        from mailup.components import Group

        # check list_data_dict
        check_group_data_dict(group_data_dict)
        list_id = group_data_dict['idList']

        new_group_data_dict = self.client.create_group(
            list_id=list_id,
            group_data_dict=group_data_dict,
        )
        new_group = Group(
            data_dict=new_group_data_dict,
            mailup_client=self.client,
            logger=self.logger
        )
        self.logger.info('Group {new_group} created successfully'.format(new_group=new_group))
        return new_group

    def get_group(self, list_id, group_id):
        all_groups = self.all_groups(list_id)
        for current_group in all_groups:
            if current_group.id == group_id:
                self.logger.debug('Group with id {group_id} found'.format(group_id=group_id))
                return current_group
        self.logger.error('Group with id {group_id} not found'.format(group_id=group_id))
        raise mailup_exceptions.GroupNotFoundException(group_id)

    def all_groups(self, list_id):
        from mailup.components import Group

        groups_list = []
        groups_data_dict = self.client.read_groups(list_id=list_id)
        for group_data_dict in groups_data_dict['Items']:
            group = Group(
                data_dict=group_data_dict,
                mailup_client=self.client,
                logger=self.logger
            )
            groups_list.append(group)
        self.logger.debug('{count} Groups founds'.format(count=len(groups_list)))
        return groups_list

    # RECIPIENT PROVIDER METHODS
    def create_recipient(self, recipient_data_dict, confirm_email=False):
        from mailup.components import Recipient

        if confirm_email:
            status = 'pending'
        else:
            status = 'subscribed'

        # check recipient_data_dict
        check_recipient_data_dict(recipient_data_dict)
        list_id = recipient_data_dict['idList']

        recipient = Recipient(
            data_dict=recipient_data_dict,
            mailup_client=self.client,
            logger=self.logger,
            status=status,
        )
        email = recipient.email
        try:
            recipient = self.get_recipient(list_id=list_id, email=email, write_log=False)
            if recipient:
                raise mailup_exceptions.RecipientAlreadyExistException(
                    list_id=list_id,
                    email=email,
                )
        except mailup_exceptions.RecipientNotFoundException:
            recipient_id = self.client.add_recipient_to_list(
                list_id=list_id,
                recipient_data_dict=recipient.recipient_data_dict,
                confirm_email=confirm_email
            )
            recipient.recipient_data_dict['idRecipient'] = recipient_id
            self.logger.info('Recipient {new_recipient} created successfully'.format(
                new_recipient=recipient
            ))
            return recipient

    def get_recipient(self, list_id, recipient_id=None, email=None, status=None, write_log=True):
        from mailup.components import Recipient

        if not recipient_id and not email:
            return self.all_recipients(list_id)

        if status == 'subscribed' or not status:
            recipient_data_paginated = self.client.get_subscribe_recipients_to_list(
                list_id=list_id,
                recipient_id=recipient_id,
                email=email,
            )
            if recipient_data_paginated and recipient_data_paginated['TotalElementsCount'] > 0:
                recipient_data_dict = recipient_data_paginated['Items'][0]
                recipient_data_dict['idList'] = list_id
                recipient = Recipient(
                    data_dict=recipient_data_dict,
                    mailup_client=self.client,
                    logger=self.logger,
                    status='subscribed',
                )
                self.logger.debug('Recipient with id {recipient_id} found'.format(
                    recipient_id=recipient.id
                ))
                return recipient
        if status == 'unsubscribed' or not status:
            recipient_data_paginated = self.client.get_unsubscribe_recipients_to_list(
                list_id=list_id,
                recipient_id=recipient_id,
                email=email,
            )
            if recipient_data_paginated and recipient_data_paginated['TotalElementsCount'] > 0:
                recipient_data_dict = recipient_data_paginated['Items'][0]
                recipient_data_dict['idList'] = list_id
                recipient = Recipient(
                    data_dict=recipient_data_dict,
                    mailup_client=self.client,
                    logger=self.logger,
                    status='unsubscribed',
                )
                self.logger.debug('Recipient with id {recipient_id} found'.format(
                    recipient_id=recipient.id
                ))
                return recipient
        if status == 'pending' or not status:
            recipient_data_paginated = self.client.get_pending_recipients_to_list(
                list_id=list_id,
                recipient_id=recipient_id,
                email=email,
            )
            if recipient_data_paginated and recipient_data_paginated['TotalElementsCount'] > 0:
                recipient_data_dict = recipient_data_paginated['Items'][0]
                recipient_data_dict['idList'] = list_id
                recipient = Recipient(
                    data_dict=recipient_data_dict,
                    mailup_client=self.client,
                    logger=self.logger,
                    status='pending',
                )
                self.logger.debug('Recipient with id {recipient_id} found'.format(
                    recipient_id=recipient.id
                ))
                return recipient
        raise mailup_exceptions.RecipientNotFoundException(
            recipient_id=recipient_id,
            email=email,
            status=status,
            write_log=write_log
        )

    def all_subscribe_recipients(self, list_id):
        from mailup.components import Recipient

        recipient_list = []
        recipients_data_dict = self.client.get_subscribe_recipients_to_list(list_id=list_id)
        if recipients_data_dict:
            for recipient_data_dict in recipients_data_dict['Items']:
                recipient_data_dict['idList'] = list_id
                recipient = Recipient(
                    data_dict=recipient_data_dict,
                    mailup_client=self.client,
                    logger=self.logger,
                    status='subscribed',
                )
                recipient_list.append(recipient)
            return recipient_list

    def all_unsubscribe_recipients(self, list_id):
        from mailup.components import Recipient

        recipient_list = []
        recipients_data_dict = self.client.get_unsubscribe_recipients_to_list(list_id=list_id)
        if recipients_data_dict:
            for recipient_data_dict in recipients_data_dict['Items']:
                recipient_data_dict['idList'] = list_id
                recipient = Recipient(
                    data_dict=recipient_data_dict,
                    mailup_client=self.client,
                    logger=self.logger,
                    status='unsubscribed',
                )
                recipient_list.append(recipient)
        return recipient_list

    def all_pending_recipients(self, list_id):
        from mailup.components import Recipient

        recipient_list = []
        recipients_data_dict = self.client.get_pending_recipients_to_list(list_id=list_id)
        if recipients_data_dict:
            for recipient_data_dict in recipients_data_dict['Items']:
                recipient_data_dict['idList'] = list_id
                recipient = Recipient(
                    data_dict=recipient_data_dict,
                    mailup_client=self.client,
                    logger=self.logger,
                    status='pending',
                )
                recipient_list.append(recipient)
        return recipient_list

    def all_recipients(self, list_id):
        subscribed_recipients = self.all_subscribe_recipients(list_id=list_id)
        unsubscribed_recipients = self.all_unsubscribe_recipients(list_id=list_id)
        pending_recipients = self.all_pending_recipients(list_id=list_id)
        all_recipient = subscribed_recipients + unsubscribed_recipients + pending_recipients
        self.logger.debug('{count} Recipient founds'.format(count=len(all_recipient)))
        return all_recipient

    # MESSAGE PROVIDER METHODS
    def create_message(
            self, message_data_dict, content='', embed=False, is_confirmation=False, tracking_info=None
    ):
        from mailup.components import Message

        check_message_data_dict(message_data_dict)

        list_id = message_data_dict['idList']

        message_data_dict['Content'] = content
        message_data_dict['Embed'] = embed
        message_data_dict['IsConfirmation'] = is_confirmation
        if tracking_info:
            message_data_dict['TrackingInfo'] = tracking_info
        else:
            message_data_dict['TrackingInfo'] = {
                'CustomParams': '',
                'Enabled': False,
                'Protocols': ['http:', 'https:', 'ftp:', 'news:'],
            }

        response = self.client.create_message(
            list_id=list_id,
            message_data_dict=message_data_dict
        )

        new_message_data_dict = response
        new_message = Message(
            data_dict=new_message_data_dict,
            mailup_client=self.client,
            logger=self.logger,
        )
        self.logger.info('Message {new_message} create successfully'.format(new_message=new_message))
        return new_message

    def get_message(self, list_id, message_id):
        from mailup.components import Message

        try:
            message_data_dict = self.client.read_message_detail(list_id, message_id)
            message = Message(
                data_dict=message_data_dict,
                mailup_client=self.client,
                logger=self.logger,
            )
            return message
        except mailup_exceptions.MailUpCallError:
            raise mailup_exceptions.MessageNotFoundException(message_id=message_id)

    def all_messages(self, list_id, status=None):
        from mailup.components import Message

        paginated_messages_data_dict = self.client.list_messages(
            list_id=list_id,
            status=status,
        )
        all_messages = []
        messages_data_dict = paginated_messages_data_dict['Items']
        for message_data_dict in messages_data_dict:
            message = Message(
                data_dict=message_data_dict,
                mailup_client=self.client,
                logger=self.logger,
            )
            all_messages.append(message)
        self.logger.debug('Messages found')
        return all_messages

    def all_published_messages(self, list_id):
        return self.all_messages(list_id=list_id, status='Online')

    def all_archived_messages(self, list_id):
        return self.all_messages(list_id=list_id, status='Archived')

    # TAG PROVIDER METHOD
    def create_tag(self, tag_data_dict):
        from mailup.components import Tag

        # check tag_data_dict
        check_tag_data_dict(tag_data_dict)
        list_id = tag_data_dict['idList']
        tag_name = tag_data_dict['Name']

        try:
            self.get_tag(list_id, tag_name=tag_name, write_log=False)
            raise mailup_exceptions.TagAlreadyExistException(
                list_id=list_id,
                tag_name=tag_name,
            )
        except mailup_exceptions.TagNotFoundException:
            response = self.client.create_tag(
                list_id=list_id,
                tag_name=tag_data_dict['Name'],
            )
            new_tag_data_dict = response
            new_tag_data_dict['idList'] = list_id
            new_tag = Tag(
                data_dict=new_tag_data_dict,
                mailup_client=self.client,
                logger=self.logger
            )
            self.logger.info('Tag {new_tag} created successfully'.format(new_tag=new_tag))
            return new_tag

    def get_tag(self, list_id, tag_id=None, tag_name=None, write_log=True):
        from mailup.components import Tag

        if not tag_id and not tag_name:
            return self.all_tags(list_id)
        tags_data_paginated = self.client.list_tags(list_id=list_id, tag_id=tag_id, tag_name=tag_name)
        if tags_data_paginated and tags_data_paginated['TotalElementsCount'] > 0:
            tag_data_dict = tags_data_paginated['Items'][0]
            tag_data_dict['idList'] = list_id
            tag = Tag(
                data_dict=tag_data_dict,
                mailup_client=self.client,
                logger=self.logger
            )
            self.logger.debug('Tag with id {tag_id} found'.format(tag_id=tag_id))
            return tag
        else:
            raise mailup_exceptions.TagNotFoundException(
                tag_id=tag_id,
                tag_name=tag_name,
                write_log=write_log
            )

    def all_tags(self, list_id):
        from mailup.components import Tag

        tags_list = []
        tags_data_dict = self.client.list_tags(list_id=list_id)
        for tag_data_dict in tags_data_dict['Items']:
            # Mailup not get (in this case) 'idList' in data_dict
            tag_data_dict['idList'] = list_id
            tag = Tag(
                data_dict=tag_data_dict,
                mailup_client=self.client,
                logger=self.logger
            )
            tags_list.append(tag)
        self.logger.debug('{count} Tags founds'.format(count=len(tags_list)))
        return tags_list

    # ATTACHMENT PROVIDER METHODS
    def all_attachments(self, list_id, message_id):
        from mailup.components import Attachment

        attachments_list = []
        attachments_data_dict = self.client.read_message_attachments(
            list_id=list_id,
            message_id=message_id,
        )
        for attachment_data_dict in attachments_data_dict:
            # Mailup not get (in this case) 'idList' and 'idMessage' in data_dict
            attachment_data_dict['idList'] = list_id
            attachment_data_dict['idMessage'] = message_id
            attachment = Attachment(
                data_dict=attachment_data_dict,
                mailup_client=self.client,
                logger=self.logger,
            )
            attachments_list.append(attachment)
        self.logger.debug('{count} Attachments founds'.format(count=len(attachments_list)))
        return attachments_list

    def get_attachment(self, list_id, message_id, file_name=None, slot=None):
        if not file_name and not slot:
            return 'Please indicate file name or file slot to find attachment'
        elif slot and not 1 <= slot <= 5:
            return 'Please indicate a slot >= 1 and <= 5'

        all_attachments = self.all_attachments(list_id, message_id)
        for attachment in all_attachments:
            if attachment.name == file_name or attachment.slot == slot:
                return attachment
        raise mailup_exceptions.AttachmentNotFoundException(slot, file_name)
